@@ -4,6 +4,8 @@ from weakref import finalize, WeakValueDictionary
 from sqlite3 import OperationalError, connect
 from typing import Iterable, Literal, Optional, Mapping
 
+from sqlite_database._debug import if_debug_print
+
 from .locals import PLUGINS_PATH
 from ._utils import (
     WithCursor,
@@ -26,7 +28,7 @@ IGNORE_TABLE_CHECKS = ("sqlite_master", "sqlite_temp_schema", "sqlite_temp_maste
 
 class Database:
     """Sqlite3 database, this provide basic integration.
-    
+
     Custom flags:
         strict : Certain actions are prevented when active, i.e, initializing nonexistent tables
         forgive: Certain actions are replaced when active, i.e, replacing .create_table to .table
@@ -53,8 +55,8 @@ class Database:
         self._config = None
         self._closed = False
         self._table_instances: dict[str, Table] = {}
-        self._strict: bool = kwargs.get('strict', True)
-        self._forgive: bool = kwargs.get('forgive', True)
+        self._strict: bool = kwargs.get("strict", True)
+        self._forgive: bool = kwargs.get("forgive", True)
         if not self._closed or self.__dict__.get("_initiated", False) is False:
             self._finalizer_fn = finalize(self, self.close)
             self._initiated = True
@@ -84,6 +86,8 @@ class Database:
         tbquery = extract_table_creations(columns)
         query = f"create table {table} ({tbquery})"
 
+        if_debug_print(query)
+
         if self._forgive and self.check_table(table):
             return self.table(table, columns)
 
@@ -116,7 +120,7 @@ class Database:
         del self._table_instances[table]
         table_._delete_hook()  # pylint: disable=protected-access
 
-    def table(self, table: str, __columns: Optional[Iterable[Column]] = None): # type: ignore
+    def table(self, table: str, __columns: Optional[Iterable[Column]] = None):  # type: ignore
         """fetch table"""
         if self._table_instances.get(table, None) is not None:
             return self._table_instances[table]
@@ -151,9 +155,9 @@ class Database:
 
     def check_table(self, table: str):
         """Check if table is exists or not."""
-        if self._path in PLUGINS_PATH:
-            plugin = self._path[2:]
-            raise ValueError(f"Plugin {plugin} must redefine check_table.")
+        # if self._path in PLUGINS_PATH:
+        #     plugin = self._path[2:]
+        #     raise ValueError(f"Plugin {plugin} must redefine check_table.")
         check_one(table)
         if table in IGNORE_TABLE_CHECKS:
             return True  # Let's return true.
@@ -197,11 +201,13 @@ class Database:
         """Rollback changes"""
         self._database.rollback()
 
-    def foreign_pragma(self, bool_state: Literal['ON', "OFF", ""] = ""):
+    def foreign_pragma(self, bool_state: Literal["ON", "OFF", ""] = ""):
         """Enable/disable foreign key pragma"""
         if bool_state not in ("ON", "OFF", ""):
             raise ValueError("Either ON/OFF for foreign key pragma.")
-        return self._database.execute(f"PRAGMA foreign_keys{'='+bool_state  if bool_state else ''}").fetchone() # pylint: disable=line-too-long
+        return self._database.execute(
+            f"PRAGMA foreign_keys{'='+bool_state  if bool_state else ''}"
+        ).fetchone()  # pylint: disable=line-too-long
 
     def optimize(self):
         """Optimize current database"""
